#!/usr/bin/perl -w
#
# $Id: Connection.pm,v 1.18 2003/06/23 01:20:15 solovam Exp $
#
# Connection class (a connection to Kismet server)
#
package Connection;

use IO::Socket;
use IO::Select;
use Gtk;
use strict;
use Observable;

my $timeout = 5;

#
# Inherit methods from Observable
#
@Connection::ISA = qw(Observable);

#
# Constructor
#
sub new
{
	# Standard prologue
	my $type = shift;
	my $self = new Observable;
	bless $self, $type;

	# Constructor arguments
	$self->{'gKismetApplication'} = shift;
	$self->{'host'} = shift;
	$self->{'port'} = shift;

	# Connect to server
	if(!$self->connect())
	{
		return(undef);
	}

	# Enable protocols
	$self->enableProtocols();

	return($self);
}

#
# Destructor
#
sub DESTROY
{
	my $self = shift;
	$self->disconnect();
}

#
# Get socket
#
sub getSocket
{
	my $self = shift;
	return $self->{'socket'};
}

#
# Connect to server
#
sub connect
{
	my $self = shift;

	$self->{'socket'} = IO::Socket::INET->new("PeerAddr" => $self->{'host'}, "PeerPort" => $self->{'port'}, "Proto" => "tcp", "Timeout" => $timeout);
	unless($self->{'socket'})
	{
		return(0);
	}

	# Register input handler
	my $inputId = Gtk::Gdk->input_add($self->getSocket()->fileno, 'read', sub {$self->readServer()}, $self->getSocket());
	$self->{'inputId'} = $inputId;

	# Is it a kismet server we connected to?
	unless($self->{'socket'}->getline() =~ /\*KISMET/)
	{
		$self->disconnect();
		return(0);
	}

	return(1);
}

#
# Disconnect from server
#
sub disconnect
{
	my $self = shift;
	if(defined $self->{'inputId'})
	{
		Gtk::Gdk->input_remove($self->{'inputId'});
	}
	if($self->{'socket'})
	{
		$self->{'socket'}->close();
	}
}

#
# Enable default set of protocols to watch
#
sub enableProtocols
{
	my $self = shift;
	my @protocols = qw(GPS INFO NETWORK CLIENT REMOVE ALERT STATUS);
	for my $p (@protocols)
	{
		$self->{'socket'}->print("!0 ENABLE $p *\n");
	}
}

#
# Eanble strings
#
sub enableStrings
{
	my $self = shift;
	$self->{'socket'}->print("!0 ENABLE STRING *\n");
}

#
# Disable strings
#
sub disableStrings
{
	my $self = shift;
	$self->{'socket'}->print("!0 REMOVE STRING\n");
}

#
# Enable packets
#
sub enablePackets
{
	my $self = shift;
	$self->{'socket'}->print("!0 ENABLE PACKET *\n");
}

#
# Disable packets
#
sub disablePackets
{
	my $self = shift;
	$self->{'socket'}->print("!0 REMOVE PACKET\n");
}

#
# Ask server to print capabilities
#
sub askCaps
{
	my $self = shift;
	my @protocols = qw(GPS INFO NETWORK CLIENT REMOVE ALERT STATUS STRING PACKET);
	for my $p (@protocols)
	{
		$self->{'socket'}->print("!0 CAPABILITY $p\n");
	}
}

#
# Read and parse input
#
sub readServer 
{
	my $self = shift;

	my $res = $self->{'socket'}->getline();
	if(!$res)
	{
		$self->notifyObservers({'updateType' => 'disconnected'});
		return;
	}
	chomp($res);
	my($cmd, $data) = ($res =~ /\*([^:]+):\s+(.*)/);

	my @fields;
	my @values;
	if($cmd eq 'GPS')
	{
		@fields = qw(lat lon alt spd fix);
		@values = Connection->oneSplit($data);
	}
	elsif($cmd eq 'INFO')
	{
		@fields = qw(networks packets crypt weak noise dropped rate signal);
		@values = Connection->oneSplit($data);
	}
	elsif($cmd eq 'NETWORK')
	{
		@fields = qw(bssid type ssid beaconinfo llcpackets datapackets cryptpackets weakpackets channel wep firsttime lasttime atype
			rangeip gpsfixed minlat minlon minalt minspd maxlat maxlon maxalt maxspd octets cloaked beaconrate maxrate
			manufkey manufscore quality signal noise bestquality bestsignal bestnoise bestlat bestlon bestalt agglat agglon
			aggalt aggpoints datasize turbocellnid turbocellmode turbocellsat carrierset);
		@values = Connection->oneSplit($data);
	}
	elsif($cmd eq 'CLIENT')
	{
		@fields = qw(bssid mac type firsttime lasttime manufkey manufscore datapackets cryptpackets weakpackets gpsfixed minlat minlon
			minalt minspd maxlat maxlon maxalt maxspd agglat agglon aggalt aggpoints maxrate quality signal noise bestquality
			bestsignal bestnoise bestlat bestlon bestalt atype ip datasize);
		@values = Connection->oneSplit($data);
	}
	elsif($cmd eq 'REMOVE')
	{
		@fields = qw(bssid);
		@values = ($data);
	}
	elsif($cmd eq 'ALERT')
	{
		@fields = qw(sec usec text);
		@values = Connection->oneSplit($data);
	}
	elsif($cmd eq 'STATUS')
	{
		@fields = qw(status);
		@values = ($data);
	}
	elsif($cmd eq 'TIME')
	{
		@fields = qw(time);
		@values = ($data);
	}
	elsif($cmd eq 'PACKET')
	{
		@fields = qw(type subtype timesec encrypted weak beaconrate sourcemac destmac bssid ssid prototype sourceip destip sourceport
			destport nbtype nbsource);
		@values = Connection->oneSplit($data);
	}
	elsif($cmd eq 'STRING')
	{
		@fields = qw(bssid sourcemac text);
		@values = Connection->oneSplit($data);
	}
	my $update = Connection->makeHash(\@fields, \@values);
	$self->notifyObservers({'updateType' => lc($cmd), 'update' => $update});
}

#
# Make hash from an array of keys and values 
#
sub makeHash
{
	shift;
	my $keys = shift;
	my $values = shift;
	my %hash;
	for(my $i = 0; $i <= $#{$keys}; $i++)
	{
			$hash{$keys->[$i]} = $values->[$i];
	}
	return \%hash;
}

#
# Get port
#
sub port
{
	my $self = shift;
	return $self->{'port'};
}

#
# Get host
#
sub host
{
	my $self = shift;
	return $self->{'host'};
}

#
# Split on space. \001 works as quotation symbol. Based on shellwords.pl
#
sub oneSplit
{
	shift;

	($_) = (@_);
	my(@words, $snippet, $field);

	s/^\s+//;
	while ($_ ne '')
	{
		$field = '';
		while(1)
		{
			if(s/^\001([^\001]*)\001//)
			{
				$snippet = $1;
			}
			elsif(s/^([^\s\001]+)//)
			{
				$snippet = $1;
			}
			else
			{
				s/^\s+//;
				last;
			}
			$field .= $snippet;
		}
		push(@words, $field);
	}
	return(@words);
}

1;
